#!/usr/bin/python3

################################################################################
#                                                                              #
# model.py                                                                     #
#                                                                              #
# Copyright (C) 2015 PJ Singh <psingh.cubic@gmail.com>                         #
#                                                                              #
################################################################################

################################################################################
#                                                                              #
# This file is part of Cubic - Custom Ubuntu ISO Creator.                      #
#                                                                              #
# Cubic is free software: you can redistribute it and/or modify                #
# it under the terms of the GNU General Public License as published by         #
# the Free Software Foundation, either version 3 of the License, or            #
# (at your option) any later version.                                          #
#                                                                              #
# Cubic is distributed in the hope that it will be useful,                     #
# but WITHOUT ANY WARRANTY; without even the implied warranty of               #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                #
# GNU General Public License for more details.                                 #
#                                                                              #
# You should have received a copy of the GNU General Public License            #
# along with Cubic.  If not, see <http://www.gnu.org/licenses/>.               #
#                                                                              #
################################################################################

import logger

import uuid

################################################################################
# General
################################################################################

terminal_pid = None
builder = None
home = None
user_name = None
user_id = None
group_id = None
password = None
uris = None
page_name = None
propagate = None
is_success_copy_original_iso_files = None
is_success_extract_squashfs = None

################################################################################
# Project
################################################################################

project_directory = None
configuration_filepath = None
original_iso_image_mount_point = None
custom_squashfs_directory = None
custom_live_iso_directory = None

################################################################################
# Original
################################################################################

original_iso_image_filepath = None
original_iso_image_filename = None
original_iso_image_directory = None
original_iso_image_volume_id = None
original_iso_image_release_name = None
original_iso_image_disk_name = None

################################################################################
# Custom
################################################################################

custom_iso_image_version_number = None
custom_iso_image_filepath = None
custom_iso_image_filename = None
custom_iso_image_directory = None
custom_iso_image_volume_id = None
custom_iso_image_release_name = None
custom_iso_image_disk_name = None

custom_iso_image_md5_sum = None
custom_iso_image_md5_filepath = None
custom_iso_image_md5_filename = None

machine_id = None

################################################################################
# General
################################################################################

def set_terminal_pid(new_terminal_pid):
    global terminal_pid
    terminal_pid = new_terminal_pid
    logger.log_data('The terminal pid is', terminal_pid)


def set_builder(new_builder):
    global builder
    builder = new_builder
    logger.log_data('The builder is', builder)


def set_home(new_home):
    global home
    home = new_home.strip() if new_home else ''
    logger.log_data('The home directory is', home)


def set_user_name(new_user_name):
    global user_name
    user_name = new_user_name.strip() if new_user_name else ''
    logger.log_data('The user name is', user_name)


def set_user_id(new_user_id):
    global user_id
    user_id = new_user_id if new_user_id else ''
    logger.log_data('The user id is', user_id)


def set_group_id(new_group_id):
    global group_id
    group_id = new_group_id if new_group_id else ''
    logger.log_data('The group id is', group_id)


def set_password(new_password):
    global password
    password = new_password.strip() if new_password else ''
    logger.log_data('The password is', password)


def set_uris(new_uris):
    global uris
    uris = new_uris
    logger.log_data('The uris is', uris)


def set_page_name(new_page_name):
    global page_name
    page_name = new_page_name.strip() if new_page_name else ''
    logger.log_data('The current page name is', page_name)


def set_propagate(new_propagate):
    global propagate
    propagate = new_propagate
    logger.log_data('Propagate assigned values to calculate dependant values?', propagate)


def set_is_success_copy_original_iso_files(new_is_success_copy_original_iso_files):
    global is_success_copy_original_iso_files
    is_success_copy_original_iso_files = new_is_success_copy_original_iso_files
    logger.log_data('Is success copy boot files', is_success_copy_original_iso_files)


def set_is_success_extract_squashfs(new_is_success_extract_squashfs):
    global is_success_extract_squashfs
    is_success_extract_squashfs = new_is_success_extract_squashfs
    logger.log_data('Is success extract squashfs?', is_success_extract_squashfs)


################################################################################
# Project
################################################################################

def set_project_directory(new_project_directory):
    global project_directory
    project_directory = new_project_directory.strip() if new_project_directory else ''.strip() if new_project_directory else ''
    logger.log_data('The project directory is', project_directory)


def set_configuration_filepath(new_configuration_filepath):
    global configuration_filepath
    configuration_filepath = new_configuration_filepath.strip() if new_configuration_filepath else ''
    logger.log_data('The configuration filepath is', configuration_filepath)


def set_original_iso_image_mount_point(new_original_iso_image_mount_point):
    global original_iso_image_mount_point
    original_iso_image_mount_point = new_original_iso_image_mount_point.strip() if new_original_iso_image_mount_point else ''
    logger.log_data('The original iso image mount point is', original_iso_image_mount_point)


def set_custom_squashfs_directory(new_custom_squashfs_directory):
    global custom_squashfs_directory
    custom_squashfs_directory = new_custom_squashfs_directory.strip() if new_custom_squashfs_directory else ''
    logger.log_data('The custom squashfs directory is', custom_squashfs_directory)


def set_custom_live_iso_directory(new_custom_live_iso_directory):
    global custom_live_iso_directory
    custom_live_iso_directory = new_custom_live_iso_directory.strip() if new_custom_live_iso_directory else ''
    logger.log_data('The custom live iso directory is', custom_live_iso_directory)


################################################################################
# Original
################################################################################

def set_original_iso_image_filepath(new_original_iso_image_filepath):
    global original_iso_image_filepath
    original_iso_image_filepath = new_original_iso_image_filepath.strip() if new_original_iso_image_filepath else ''
    logger.log_data('The original iso image filepath is', original_iso_image_filepath)


def set_original_iso_image_filename(new_original_iso_image_filename):
    global original_iso_image_filename
    original_iso_image_filename = new_original_iso_image_filename.strip() if new_original_iso_image_filename else ''
    logger.log_data('The original iso image filename is', original_iso_image_filename)


def set_original_iso_image_directory(new_original_iso_image_directory):
    global original_iso_image_directory
    original_iso_image_directory = new_original_iso_image_directory.strip() if new_original_iso_image_directory else ''
    logger.log_data('The original iso image directory is', original_iso_image_directory)


def set_original_iso_image_volume_id(new_original_iso_image_volume_id):
    global original_iso_image_volume_id
    original_iso_image_volume_id = new_original_iso_image_volume_id.strip() if new_original_iso_image_volume_id else ''
    logger.log_data('The original iso image volume_id is', original_iso_image_volume_id)


def set_original_iso_image_release_name(new_original_iso_image_release_name):
    global original_iso_image_release_name
    original_iso_image_release_name = new_original_iso_image_release_name.strip() if new_original_iso_image_release_name else ''
    logger.log_data('The original iso image release name is', original_iso_image_release_name)


def set_original_iso_image_disk_name(new_original_iso_image_disk_name):
    global original_iso_image_disk_name
    original_iso_image_disk_name = new_original_iso_image_disk_name.strip() if new_original_iso_image_disk_name else ''
    logger.log_data('The original iso image disk name is', original_iso_image_disk_name)


################################################################################
# Custom
################################################################################

def set_custom_iso_image_version_number(new_custom_iso_image_version_number):
    global custom_iso_image_version_number
    custom_iso_image_version_number = new_custom_iso_image_version_number.strip() if new_custom_iso_image_version_number else ''
    logger.log_data('The custom iso image version number is', custom_iso_image_version_number)


def set_custom_iso_image_filepath(new_custom_iso_image_filepath):
    global custom_iso_image_filepath
    custom_iso_image_filepath = new_custom_iso_image_filepath.strip() if new_custom_iso_image_filepath else ''
    logger.log_data('The custom iso image filepath is', custom_iso_image_filepath)


def set_custom_iso_image_filename(new_custom_iso_image_filename):
    global custom_iso_image_filename
    custom_iso_image_filename = new_custom_iso_image_filename.strip() if new_custom_iso_image_filename else ''
    logger.log_data('The custom iso image filename is', custom_iso_image_filename)


def set_custom_iso_image_directory(new_custom_iso_image_directory):
    global custom_iso_image_directory
    custom_iso_image_directory = new_custom_iso_image_directory.strip() if new_custom_iso_image_directory else ''
    logger.log_data('The custom iso image directory is', custom_iso_image_directory)


def set_custom_iso_image_volume_id(new_custom_iso_image_volume_id):
    global custom_iso_image_volume_id
    custom_iso_image_volume_id = new_custom_iso_image_volume_id.strip() if new_custom_iso_image_volume_id else ''
    logger.log_data('The custom iso image volume_id is', custom_iso_image_volume_id)


def set_custom_iso_image_release_name(new_custom_iso_image_release_name):
    global custom_iso_image_release_name
    custom_iso_image_release_name = new_custom_iso_image_release_name.strip() if new_custom_iso_image_release_name else ''
    logger.log_data('The custom iso image release name is', custom_iso_image_release_name)


def set_custom_iso_image_disk_name(new_custom_iso_image_disk_name):
    global custom_iso_image_disk_name
    custom_iso_image_disk_name = new_custom_iso_image_disk_name.strip() if new_custom_iso_image_disk_name else ''
    logger.log_data('The custom iso image disk name is', custom_iso_image_disk_name)


def set_custom_iso_image_md5_sum(new_custom_iso_image_md5_sum):
    global custom_iso_image_md5_sum
    custom_iso_image_md5_sum = new_custom_iso_image_md5_sum.strip() if new_custom_iso_image_md5_sum else ''
    logger.log_data('The custom iso image md5 sum is', custom_iso_image_md5_sum)


def set_custom_iso_image_md5_filepath(new_custom_iso_image_md5_filepath):
    global custom_iso_image_md5_filepath
    custom_iso_image_md5_filepath = new_custom_iso_image_md5_filepath.strip() if new_custom_iso_image_md5_filepath else ''
    logger.log_data('The custom iso image md5 filepath is', custom_iso_image_md5_filepath)


def set_custom_iso_image_md5_filename(new_custom_iso_image_md5_filename):
    global custom_iso_image_md5_filename
    custom_iso_image_md5_filename = new_custom_iso_image_md5_filename.strip() if new_custom_iso_image_md5_filename else ''
    logger.log_data('The custom iso image md5 filename is', custom_iso_image_md5_filename)


def get_machine_id():
    global machine_id
    try:
        machine_id
    except NameError:
        logger.log_step('Calculating a new machine id')
        machine_id = str(uuid.uuid4()).replace('-', '')
    return machine_id


def set_machine_id(new_machine_id):
    global machine_id
    machine_id = new_machine_id
    # logger.log_data('The machine id is', machine_id)
